const std = @import("std");
const SDL = @import("sdl2");

const MenuSelection = @import("MenuSelection.zig");
const MenuTab = @import("MenuTab.zig");

const State = @import("flow.zig").State;

const Self = @This();

tab_list: [2]MenuTab = [2]MenuTab{
    MenuTab.init("1P", &.{
        MenuSelection.init("Play", play),
        MenuSelection.init("Print", print),
    }),
    MenuTab.init("Options", &.{
        MenuSelection.init("Play", play),
        MenuSelection.init("Print", print),
    }),
},

// Current tab and selection index
tab: usize = 0,
sel: usize = 0,

state: State = State.main_menu,

holding_down: bool = false,
holding_enter: bool = false,

fn getSel(self: *Self) *MenuSelection {
    return &self.tab_list[self.tab].contents[self.sel];
}

fn getTab(self: *Self) *MenuTab {
    return &self.tab_list[self.tab];
}

pub fn init() Self {
    return Self{};
}

pub fn tick(self: *Self) State {
    const sel = self.getSel();
    const tab = self.getTab();
    var key_state = SDL.SDL_GetKeyboardState(null);
    if (key_state[SDL.SDL_SCANCODE_DOWN] == 1 and !self.holding_down) {
        self.holding_down = true;
        self.actionSelDown();
    }
    if (key_state[SDL.SDL_SCANCODE_DOWN] == 0) {
        self.holding_down = false;
    }
    if (key_state[SDL.SDL_SCANCODE_RETURN] == 1 and !self.holding_enter) {
        self.holding_enter = true;
        self.actionSelect();
    }
    if (key_state[SDL.SDL_SCANCODE_RETURN] == 0) {
        self.holding_enter = false;
    }
    std.debug.print(
        \\Tab: {s}
        \\Selection: {s}
        \\
    , .{ tab.*.name, sel.*.name });
    return self.state;
}

fn actionSelDown(self: *Self) void {
    self.sel += 1;
    self.sel %= self.tab_list[self.tab].contents.len;
}

fn actionSelect(self: *Self) void {
    const action = self.getSel().*.action;
    @call(.{}, action, .{self});
}

fn play(self: *Self) void {
    self.state = State.game;
}

fn print(self: *Self) void {
    _ = self;
    std.debug.print("Print\n", .{});
}
